from django.conf import settings
from pyepayco import epayco

options = {
    'apiKey': settings.EPAYCO_PUBLIC_KEY,
    'privateKey': settings.EPAYCO_SECRET_KEY,
    'test': settings.DEBUG_EPAYCO,
    'lenguage': 'EN',
}

epayco_client = epayco.Epayco(options)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2018-10-11 19:22
from __future__ import unicode_literals

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('depayco', '0003_auto_20181011_1421'),
    ]

    operations = [
        migrations.AddField(
            model_name='subscription',
            name='current_period_end',
            field=models.DateTimeField(default=django.utils.timezone.now),
            preserve_default=False,
        ),
    ]



from depayco import epayco_client
from depayco.models import Customer


def create(credit_card_token, name, email, phone, user_id, identification_type, identification_number, default=True):
    """

    :param credit_card_token:
    :param name:
    :param email:
    :param phone:
    :param default:
    :param user_id:
    :param identification: {'type': 'NIT', 'number': '3434566'}
    :return:
    """
    customer_info = {
        'token_card': credit_card_token,
        'name': name,
        'email': email,
        'phone': phone,
        'default': default
    }

    customer = epayco_client.customer.create(customer_info)

    return sync_customer_from_epayco_data(
        epayco_customer_data=customer['data'],
        credit_card_token=credit_card_token,
        user_id=user_id,
        identification_type=identification_type,
        identification_number=identification_number,
    )


def update(customer_id, customer_data):
    raise NotImplementedError


def sync_customer_from_epayco_data(epayco_customer_data, credit_card_token, user_id, identification_type, identification_number):
    defaults = dict(
        user_id=user_id,
        credit_card_token=credit_card_token,
        identification_type=identification_type,
        identification_number=identification_number,
    )

    customer, created = Customer.objects.get_or_create(
        id=epayco_customer_data['customerId'],
        defaults=defaults,
    )

    return customer




from datetime import datetime

from depayco import epayco_client
from depayco.models import Subscription


def create_and_charge(plan_id, customer_id, credit_card_token, identification_type, identification_number):
    subscription_data = {
        'id_plan': plan_id,
        'customer': customer_id,
        'token_card': credit_card_token,
        'doc_type': identification_type,
        'doc_number': identification_number,
    }

    epayco_subscription = epayco_client.subscriptions.create(subscription_data)
    epayco_subscription_charge = epayco_client.subscriptions.charge(subscription_data)

    return sync_subscription_from_epayco_data(
        epayco_subscription_data=epayco_subscription,
        epayco_subscription_data_charge=epayco_subscription_charge,
        customer_id=customer_id,
        plan_id=plan_id,
    )


def cancel(subscription_id):
    epayco_client.subscriptions.cancel(subscription_id)

    return sync_cancel_subscription(subscription_id=subscription_id)


def has_active_subscription(subscription_id):
    raise NotImplementedError


def intervals_paid(customer_id):
    raise NotImplementedError


def sync_subscription_from_epayco_data(epayco_subscription_data, epayco_subscription_data_charge, customer_id, plan_id):
    start_day, start_month, start_year = epayco_subscription_data['current_period_start'].split('-')
    end_day, end_month, end_year = epayco_subscription_data_charge['subscription']['periodEnd'].split('-')

    defaults = dict(
        customer_id=customer_id,
        plan_id=plan_id,
        current_period_start='{0}-{1}-{2}'.format(start_year, start_month, start_day),
        current_period_end='{0}-{1}-{2}'.format(end_year, end_month, end_day),
    )

    subscription, created = Subscription.objects.get_or_create(
        id=epayco_subscription_data['id'],
        defaults=defaults,
    )

    if created is False:
        subscription.cancel_at_period_end = False
        subscription.save(update_fields=['cancel_at_period_end'])

    return subscription


def sync_cancel_subscription(subscription_id):
    return Subscription.objects.filter(id=subscription_id).update(
        cancel_at_period_end=True, canceled_at=datetime.today()
    )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2018-10-11 21:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('depayco', '0004_subscription_current_period_end'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subscription',
            name='current_period_end',
            field=models.DateField(),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='current_period_start',
            field=models.DateField(),
        ),
    ]



from django.contrib.auth.decorators import login_required
from django.shortcuts import redirect, render
from django.urls import reverse
from django.utils.decorators import method_decorator
from django.views.generic import FormView, View
from django.contrib import messages

from depayco.actions import credit_cards, customers, subscriptions
from depayco.addons.decorators import already_subscribed
from depayco.addons.forms import ChargeInformationForm


@method_decorator(already_subscribed, name='dispatch')
@method_decorator(login_required, name='dispatch')
class EpaycoView(FormView):
    template_name = 'epayco-form.html'
    form_class = ChargeInformationForm


class ProcessPayment(View):
    @method_decorator(login_required)
    def post(self, request):
        try:
            token = credit_cards.tokenize(
                credit_card_number=request.POST.get('credit_card_number'),
                exp_year=request.POST.get('credit_card_exp_year'),
                exp_month=request.POST.get('credit_card_exp_month'),
                cvc=request.POST.get('cvc'),
            )['data']['id']

            customer = customers.create(
                credit_card_token=token,
                name=request.POST.get('name'),
                email=request.POST.get('email'),
                phone=request.POST.get('phone'),
                user_id=request.user.id,
                identification_type=request.POST.get('identification_type'),
                identification_number=request.POST.get('identification_number'),
            )

            subscriptions.create_and_charge(
                plan_id='pronostico_cambiario',
                customer_id=customer.id,
                credit_card_token=token,
                identification_type=request.POST.get('identification_type'),
                identification_number=request.POST.get('identification_number'),
            )
        except Exception as e:
            messages.add_message(
                request,
                messages.WARNING,
                'No se pudo procesar su pago, verifique que la información ingresada sea correcta',
            )
            form = ChargeInformationForm(request.POST)

            return render(
                request, 'epayco-form.html', {
                    'form': form
                }
            )

        return redirect(reverse('home'))


class CancelSubscription(View):
    @method_decorator(login_required)
    def get(self, request):
        try:
            subscriptions.cancel(subscription_id=request.GET.get('subscription_id'))
        except:
            messages.add_message(
                request,
                messages.WARNING,
                'No se pudo procesar su cancelación, vuelva a intentarlo más tarde.',
            )

        return redirect(reverse('my_subscription'))


class ReSubscribe(View):
    @method_decorator(already_subscribed)
    @method_decorator(login_required)
    def get(self, request):
        return render(
            request=request, template_name='re-subscribe.html'
        )

    @method_decorator(login_required)
    def post(self, request):
        try:
            subscriptions.create_and_charge(
                plan_id='pronostico_cambiario',
                customer_id=request.user.customer.id,
                credit_card_token=request.user.customer.credit_card_token,
                identification_type=request.user.customer.identification_type,
                identification_number=request.user.customer.identification_number,
            )
        except:
            messages.add_message(
                request,
                messages.WARNING,
                'No se pudo procesar su re-subscripción, vuelva a intentarlo más tarde.',
            )

        return redirect(reverse('my_subscription'))



from django.conf import settings
from django.db import models

from depayco import epayco_client
from depayco.addons.choices import INTERVAL, IDENTIFICATION_TYPE


class Customer(models.Model):
    id = models.CharField(
        max_length=20, primary_key=True, unique=True, help_text='Example: 9xRxhaJ2YmLTkT5uz'
    )
    user = models.OneToOneField(
        settings.AUTH_USER_MODEL, on_delete=models.CASCADE
    )
    credit_card_token = models.CharField(
        max_length=20, help_text='Example: eXj5Wdqgj7xzvC7AR'
    )
    identification_type = models.CharField(
        max_length=5, choices=IDENTIFICATION_TYPE,
    )
    identification_number = models.CharField(
        max_length=200
    )
    delinquent = models.BooleanField(
        default=False
    )
    created = models.DateTimeField(
        auto_now_add=True
    )

    def __str__(self):
        return self.user.email

    def update_epayco_customer(self, update_customer_info):
        customer = epayco_client.customer.update(self.id, update_customer_info)

        return customer

    def update_epayco_credit_card(self):
        raise NotImplementedError


class Plan(models.Model):
    id = models.CharField(
        max_length=30, primary_key=True, unique=True, help_text='Example: unity_3d_course',
    )
    name = models.CharField(
        max_length=150
    )
    description = models.TextField(

    )
    amount = models.DecimalField(
        decimal_places=2, max_digits=9
    )
    currency = models.CharField(
        max_length=3,
    )
    interval = models.CharField(
        max_length=5, choices=INTERVAL,
    )
    interval_count = models.IntegerField(

    )
    trial_days = models.IntegerField(
        default=0,
    )
    created = models.DateTimeField(
        auto_now_add=True
    )

    def __str__(self):
        return self.name

    def update_epayco_plan(self, update_plan_info):
        plan = epayco_client.plan.update(self.id, update_plan_info)

        return plan


class Subscription(models.Model):
    id = models.CharField(
        max_length=20, primary_key=True, unique=True, help_text='Example: efPXtZ5r4nZRoPtjZ'
    )
    customer = models.ForeignKey(
        'depayco.Customer', on_delete=models.CASCADE,
    )
    plan = models.ForeignKey(
        'depayco.Plan', on_delete=models.CASCADE,
    )
    is_active = models.BooleanField(
        default=True,
    )
    current_period_start = models.DateField(

    )
    current_period_end = models.DateField(

    )
    cancel_at_period_end = models.BooleanField(
        default=False
    )
    canceled_at = models.DateTimeField(
        null=True, blank=True
    )
    created = models.DateTimeField(
        auto_now_add=True
    )

    def __str__(self):
        return '{0} {1}'.format(self.customer, self.plan)

    def cancel_epayco_subscription(self):
        subscription = epayco_client.subscriptions.cancel(self.id)

        return subscription



from depayco import epayco_client
from depayco.models import Plan


def create(plan_data):
    epayco_plan = epayco_client.plan.create(plan_data)

    return sync_plan_from_epayco_data(epayco_plan['data'])


def delete(plan_id):
    epayco_client.plan.delete(plan_id)

    return Plan.objects.get(id=plan_id).delete()


def update(plan_id, plan_data):
    copy_plan_data = remove_plan_key(plan_data=plan_data)
    epayco_client.plan.update(plan_id, copy_plan_data)

    return update_plan(plan_data=plan_data)


def sync_plan_from_epayco_data(epayco_plan_data):
    defaults = dict(
        name=epayco_plan_data['name'],
        description=epayco_plan_data['description'],
        amount=epayco_plan_data['amount'],
        currency=epayco_plan_data['currency'],
        interval=epayco_plan_data['interval'],
        interval_count=epayco_plan_data['interval_count'],
        trial_days=epayco_plan_data['trial_days'],
    )

    plan, created = Plan.objects.get_or_create(
        id=epayco_plan_data['id_plan'],
        defaults=defaults,
    )

    return plan


def update_plan(plan_data):
    plan = Plan.objects.get(id=plan_data['id_plan'])

    plan.name = plan_data['name']
    plan.description = plan_data['description']
    plan.amount = plan_data['amount']
    plan.currency = plan_data['currency']
    plan.interval = plan_data['interval']
    plan.interval_count = plan_data['interval_count']
    plan.trial_days = plan_data['trial_days']

    plan.save()


def remove_plan_key(plan_data, key='id_plan'):
    copy_plan_data = dict(plan_data)
    del copy_plan_data[key]

    return copy_plan_data



from django.conf import settings
from django.core.management import BaseCommand

from depayco import epayco_client
from depayco.actions import plans
from depayco.models import Plan


class Command(BaseCommand):
    def handle(self, *args, **options):
        try:
            if len(settings.PLANS) > 0:
                for settings_plan in settings.PLANS:
                    response = epayco_client.plan.get(settings_plan['id_plan'])

                    if 'message' in response and response['message'] == 'Plan no encontrado':
                        plans.create(plan_data=settings_plan)

                        self.stdout.write(self.style.SUCCESS(
                            f'Successfully create plan {settings_plan["id_plan"]} on epayco and local DB'
                        ))

                    elif response['success'] and not Plan.objects.filter(id=settings_plan['id_plan']).exists():
                        plans.sync_plan_from_epayco_data(epayco_plan_data=settings_plan)

                        self.stdout.write(self.style.SUCCESS(
                            f'Successfully create plan {settings_plan["id_plan"]} on local DB (from existing data on epayco)'
                        ))

                    elif response['success']:
                        plans.update(plan_id=settings_plan['id_plan'], plan_data=settings_plan)

                        self.stdout.write(self.style.SUCCESS(
                            f'Successfully update plan {settings_plan["id_plan"]} on epayco and local DB'
                        ))

        except AttributeError:
            self.stdout.write(self.style.WARNING('Is needed configure PLANS in settings.py'))



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2018-10-09 16:59
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Customer',
            fields=[
                ('id', models.CharField(help_text='Example: 9xRxhaJ2YmLTkT5uz', max_length=20, primary_key=True, serialize=False, unique=True)),
                ('credit_card_token', models.CharField(help_text='Example: eXj5Wdqgj7xzvC7AR', max_length=20)),
                ('identification_type', models.CharField(choices=[('CC', 'Cédula de ciudadanía'), ('CE', 'Cédula de extranjería '), ('NIT', 'Número de Identificación Tributaria'), ('DNI', 'Documento nacional de identidad')], max_length=5)),
                ('identification_number', models.CharField(max_length=200)),
                ('delinquent', models.BooleanField(default=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Plan',
            fields=[
                ('id', models.CharField(help_text='Example: unity_3d_course', max_length=30, primary_key=True, serialize=False, unique=True)),
                ('name', models.CharField(max_length=150)),
                ('description', models.TextField()),
                ('amount', models.DecimalField(decimal_places=2, max_digits=9)),
                ('currency', models.CharField(max_length=3)),
                ('interval', models.CharField(choices=[('day', 'Day'), ('week', 'Week'), ('month', 'Month'), ('year', 'Year')], max_length=5)),
                ('interval_count', models.IntegerField()),
                ('trial_days', models.IntegerField(default=0)),
                ('created', models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name='Subscription',
            fields=[
                ('id', models.CharField(help_text='Example: efPXtZ5r4nZRoPtjZ', max_length=20, primary_key=True, serialize=False, unique=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('customer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='depayco.Customer')),
                ('plan', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='depayco.Plan')),
            ],
        ),
    ]



from depayco import epayco_client


def tokenize(credit_card_number, exp_year, exp_month, cvc):
    if len(exp_month) == 1:
        exp_month = '0{exp_month}'.format(exp_month=exp_month)

    credit_card_data = {
        'card[number]': credit_card_number.replace(' ', ''),
        'card[exp_year]': exp_year.replace(' ', ''),
        'card[exp_month]': exp_month.replace(' ', ''),
        'card[cvc]': cvc.replace(' ', ''),
    }

    return epayco_client.token.create(credit_card_data)



INTERVAL = (
    ('day', 'Day'),
    ('week', 'Week'),
    ('month', 'Month'),
    ('year', 'Year'),
)

IDENTIFICATION_TYPE = (
    ('cc', 'Cédula de ciudadanía'),
    ('ce', 'Cédula de extranjería '),
    ('nit', 'Número de Identificación Tributaria'),
    ('dni', 'Documento nacional de identidad'),
)



from django.shortcuts import redirect
from django.urls import reverse


def subscription_required(function):
    """
    """
    def return_function(request, *args, **kwargs):
        if request.user.is_staff:
            return function(request, *args, **kwargs)

        if hasattr(request.user, 'customer'):
            if not request.user.customer.subscription_set.filter(is_active=True).exists():
                return redirect(reverse('depayco:re_subscribe'))
            else:
                return function(request, *args, **kwargs)

        elif request.user.is_authenticated:
            return redirect(reverse('depayco:epayco_form'))
        else:
            return redirect('login')

    return return_function


def already_subscribed(function):
    """
    """
    def return_function(request, *args, **kwargs):
        if request.user.is_staff:
            return function(request, *args, **kwargs)

        if hasattr(request.user, 'customer'):
            if request.user.customer.subscription_set.filter(is_active=True).exists():
                return redirect(reverse('home'))
            else:
                return function(request, *args, **kwargs)

        elif request.user.is_authenticated:
            return function(request, *args, **kwargs)
        else:
            return redirect('login')

    return return_function



from django.conf.urls import url

from .views import EpaycoView, ProcessPayment, CancelSubscription, ReSubscribe

urlpatterns = [
    url(r'^epayco-form/$', EpaycoView.as_view(), name='epayco_form'),
    url(r'^process-payment/$', ProcessPayment.as_view(), name='process_payment'),
    url(r'^cancel-subscription/$', CancelSubscription.as_view(), name='cancel_subscription'),
    url(r'^re-subscribe/$', ReSubscribe.as_view(), name='re_subscribe'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2018-10-09 19:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('depayco', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='subscription',
            name='is_active',
            field=models.BooleanField(default=True),
        ),
    ]



from datetime import date

from django.core.management import BaseCommand

from depayco import epayco_client
from depayco.models import Subscription


# TODO: add output messages
class Command(BaseCommand):
    def handle(self, *args, **options):
        active_subscriptions = Subscription.objects.filter(is_active=True)

        for active_subscription in active_subscriptions:
            epayco_subscription = epayco_client.subscriptions.get(active_subscription.id)

            start_day, start_month, start_year = epayco_subscription['current_period_start'].split('-')
            end_day, end_month, end_year = epayco_subscription['current_period_end'].split('-')

            epayco_subscription.current_period_start = '{0}-{1}-{2}'.format(start_year, start_month, start_day)
            epayco_subscription.current_period_end = '{0}-{1}-{2}'.format(end_year, end_month, end_day)

            # TODO: verify if dates changed

            epayco_subscription.save(update_fields=['current_period_start', 'current_period_end'])

        canceled_subscriptions = Subscription.objects.filter(cancel_at_period_end=True)

        for canceled_subscription in canceled_subscriptions:
            if canceled_subscription.current_period_end <= date.today():
                canceled_subscription.is_active = True

                canceled_subscription.save(update_fields=['is_active'])



from django import forms

from depayco.addons.choices import IDENTIFICATION_TYPE


class ChargeInformationForm(forms.Form):
    name = forms.CharField(
        max_length=50,
    )
    email = forms.EmailField(

    )
    credit_card_number = forms.CharField(
        max_length=50
    )
    cvc = forms.CharField(
        max_length=4,
    )
    credit_card_exp_month = forms.CharField(
        max_length=2,
    )
    credit_card_exp_year = forms.CharField(
        max_length=4,
    )
    phone = forms.CharField(
        max_length=10,
    )
    identification_type = forms.ChoiceField(
        choices=IDENTIFICATION_TYPE,
    )
    identification_number = forms.CharField(
        max_length=200,
    )



import setuptools

with open('README.md', 'r') as readme:
    long_description = readme.read()

setuptools.setup(
    name='depayco',
    version='1.1.1',
    license='GPL',
    author='Alonso',
    author_email='alons45@gmail.com',
    description='Django integration with ePayco (epayco.co)',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/ma1onso/depayco',
    packages=setuptools.find_packages(),
    install_requires=[
        'pyepayco==1.0.0',
        # TODO: don't repeat dependencies from pyepayco
        'requests >= 2.4.3',
        'pycrypto >= 2.3'
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Framework :: Django',
        'Environment :: Web Environment',
    ],
    python_requires='>=3.6',
)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2018-10-11 19:21
from __future__ import unicode_literals

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('depayco', '0002_subscription_is_active'),
    ]

    operations = [
        migrations.AddField(
            model_name='subscription',
            name='current_period_start',
            field=models.DateTimeField(default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='customer',
            name='identification_type',
            field=models.CharField(choices=[('cc', 'Cédula de ciudadanía'), ('ce', 'Cédula de extranjería '), ('nit', 'Número de Identificación Tributaria'), ('dni', 'Documento nacional de identidad')], max_length=5),
        ),
    ]
